@@ -3,30 +3,20 @@
 #include <taskflow/taskflow.hpp>
 
 int main() {
-  
+
   tf::Executor executor;
-  
+
   // create asynchronous tasks from the executor
   // (using executor as a thread pool)
-  tf::Future<std::optional<int>> future1 = executor.async([](){
+  std::future<int> fu = executor.async([](){
     std::cout << "async task 1 returns 1\n";
     return 1;
   });
 
-  executor.silent_async([](){  // silent async task doesn't return
+  executor.silent_async([](){  // silent async task doesn't return any future object
     std::cout << "async task 2 does not return (silent)\n";
   });
 
-  // create asynchronous tasks with names (for profiling)
-  executor.named_async("async_task", [](){
-    std::cout << "named async task returns 1\n";
-    return 1;
-  });
-
-  executor.named_silent_async("silent_async_task", [](){
-    std::cout << "named silent async task does not return\n";
-  });
-
   executor.wait_for_all();  // wait for the two async tasks to finish
 
   // create asynchronous tasks from a subflow
@@ -55,3 +45,10 @@ int main() {
   return 0;
 }
 
+
+
+
+
+
+
+
